function [X s R M C var_D] = EM_PND(s, R, M, D, W, th, iter_max)
% function [X s R M C var_D] = EM_PND(s, R, M, D, W, th, iter_max)
%
% Fit PND to a given data using EM algorithm
%
% Common:
%     s: Scales       (1 x n)
%     R: Rotations    (3 x 3 x n)
%     M: Mean shape   (3 x p)
%     
% Inputs:
%     D: Observations                                 (3 x p x n)
%     W: Weights - observed (true) or missing (false) (3 x p x n)
%     th: Threshold for iteration
%     iter_max: Maximum number of iteration
%
% Outputs:
%     X: Reconstructed shapes (3 x p x n)
%     C: Shape covariance     (3p x 3p)
%     var_D: Noise variance
%
% Ref: Minsik Lee, Jungchan Cho, Chong-Ho Choi, and Songhwai Oh,
% "Procrustean Normal Distribution for Non-Rigid Structure from Motion,"
% CVPR 2013, Portland, Oregon, June 23-28, 2013.
%
% Authors: Minsik Lee (mlee.paper@gmail.com), Jungchan Cho (jungchan.cho@gmail.com)
% Last update: 2013-08-08
% License: GPLv3

%
% Copyright (C) 2013 Minsik Lee, Jungchan Cho
% This file is part of NRSfM_PND.
%
% NRSfM_PND is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% NRSfM_PND is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with NRSfM_PND.  If not, see <http://www.gnu.org/licenses/>.


% initial values and parameters
C_init = 1e-3;
var_D = 1e-6;
v_comp = 2;

if nargin < 6 || isempty(th)
    th = 1e-10;
end
if nargin < 7 || isempty(iter_max)
    iter_max = 1e3;
end

[k p nSample] = size(D);

pq = p-1;
pdim = k*pq;
ndim = k+1;
cdim = pdim-ndim;
[Qt ~] = qr(ones(p, 1));
Qt = Qt(:, 2:end);
QT = kron(Qt, eye(k));

WW = zeros(pdim, pdim, nSample);
sW = zeros(k, nSample);
for i=1:nSample
    tW = W(:, :, i);
    t1 = 1./sum(tW, 2);
    t1(isinf(t1)) = 0;
    sW(:, i) = t1;
    t2 = kron(ones(p, 1), diag(sqrt(t1)));
    t2 = t2(tW, :);
    tP = QT(tW, :);
    t2 = tP'*t2;
    WW(:, :, i) = tP'*tP-t2*t2';
end
nW = sum(W(:));
dW = nW-sum(sW(:) > 0);

sR = zeros(k, k, nSample);
for i=1:nSample
    sR(:, :, i) = s(i)*R(:, :, i);
end
sRX = zeros(k, pq, nSample);

% Exclude translation bases
M = M*Qt;
X = zeros(k, pq, nSample);
wX = zeros(k, pq, nSample);
qD = reshape(ipermute(reshape(reshape(permute(D, [1 3 2]), k*nSample, p)*Qt, k, nSample, pq), [1 3 2]), pdim, nSample);

% Caculate initial Q
temp = zeros(k, pq, k);
temp(1, :, 2) = -M(3, :);
temp(1, :, 3) = M(2, :);
temp(2, :, 1) = M(3, :);
temp(2, :, 3) = -M(1, :);
temp(3, :, 1) = -M(2, :);
temp(3, :, 2) = M(1, :);
[Q ~] = qr([reshape(temp, pdim, k) M(:)]);
Q = Q(:, ndim+1:end);

% Initial shape covariance
Cp = eye(cdim)*C_init;
C_X = zeros(pdim, pdim, nSample);

count = 0;
cost = inf;
eM = inf;
while eM > th && count < iter_max
    pcost = cost;
    pM = M;

    %%% E-Step %%%
    tC = Q/(Cp/var_D)*Q';
    for i=1:nSample
        C_X(:, :, i) = inv(reshape(sR(:, :, i)'*reshape(reshape(sR(:, :, i)'*reshape(tC, k, []), pdim, pdim)', k, []), pdim, pdim) + WW(:, :, i));
        temp = C_X(:, :, i)*qD(:, i);
        X(:, :, i) = reshape(temp, k, pq);
        wX(:, :, i) = reshape(WW(:, :, i)*temp, k, pq);
    end
    C_X = (C_X+permute(C_X, [2 1 3]))*(var_D/2);
    
    %%% M-Step %%%

    % M
    M = zeros(k, pq);
    for i=1:nSample
        M = M+sR(:, :, i)*X(:, :, i);
    end
    M = M/norm(M(:));

    % R, s
    tC = zeros(pdim, pdim);
    for i=1:nSample
        [U S V] = svd(M*X(:,:,i)','econ');
        R(:,:,i) = U*V';
        s(i) = 1/sum(diag(S));
        
        sR(:, :, i) = s(i)*R(:, :, i);
        sRX(:, :, i) = sR(:, :, i)*X(:, :, i);
        tC = tC + reshape(sR(:, :, i)*reshape(reshape(sR(:, :, i)*reshape(C_X(:, :, i), k, []), pdim, pdim)', k, []), pdim, pdim);
    end
    
    % Q
    temp = zeros(k, pq, k);
    temp(1, :, 2) = -M(3, :);
    temp(1, :, 3) = M(2, :);
    temp(2, :, 1) = M(3, :);
    temp(2, :, 3) = -M(1, :);
    temp(3, :, 1) = -M(2, :);
    temp(3, :, 2) = M(1, :);
    [Q ~] = qr([reshape(temp, pdim, k) M(:)]);
    Q = Q(:, ndim+1:end);
    
    % Cp
    vX = reshape(sRX, pdim, nSample)-repmat(M(:), 1, nSample);
    temp = Q'*(vX*vX'+tC)*Q;
    Cp = (temp + temp')/(2*nSample);
    Cp = Cp + 1e-15*trace(Cp)*eye(cdim); % avoiding singularity

    % var_D
    var_D = norm(qD(:)-wX(:))^2/dW + WW(:)'*C_X(:)/dW;
    var_D = var_D*v_comp; % compensation
    
    % Covergence check and calculating the cost value
    eM = norm(M(:) - pM(:))^2;
    cost = sum(log(eig(Cp))) + log(var_D)*nW/nSample;
    
    count = count + 1;
    disp(['EM-PND ' num2str(count) ' : eM = ' num2str(eM) ' / var_D = ' num2str(var_D) ' / d_cost = ' num2str(pcost - cost) ' / cost = ' num2str(cost)]);
end

% Restore translation bases
M = M*Qt';
X = ipermute(reshape(reshape(permute(X, [1 3 2]), [], pq)*Qt', k, nSample, p), [1 3 2]);
tt = QT*Q;
C = tt*Cp*tt';
C = (C+C')/2;


end

