function [X s R M C var_D] = NRSfM_PND(D, W)
% function [X s R M C var_D] = NRSfM_PND(D, W)
%
% Solve NRSfM problem based on PND
%
% Inputs:
%     D: Observations                                 (3 x p x n)
%     W: Weights - observed (true) or missing (false) (3 x p x n)
%
% Outputs:
%     X: Reconstructed shapes (3 x p x n)
%     s: Scales               (1 x n)
%     R: Rotations            (3 x 3 x n)
%     M: Mean shape           (3 x p)
%     C: Shape covariance     (3p x 3p)
%     var_D: Noise variance
%
% Ref: Minsik Lee, Jungchan Cho, Chong-Ho Choi, and Songhwai Oh,
% "Procrustean Normal Distribution for Non-Rigid Structure from Motion,"
% CVPR 2013, Portland, Oregon, June 23-28, 2013.
%
% Authors: Minsik Lee (mlee.paper@gmail.com), Jungchan Cho (jungchan.cho@gmail.com)
% Last update: 2013-08-08
% License: GPLv3

%
% Copyright (C) 2013 Minsik Lee, Jungchan Cho
% This file is part of NRSfM_PND.
%
% NRSfM_PND is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% NRSfM_PND is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with NRSfM_PND.  If not, see <http://www.gnu.org/licenses/>.


[k p nSample] = size(D);

% Normalize
D = (D - repmat(sum(D.*W, 2)./sum(W, 2), 1, p)).*W;
D(isnan(D)) = 0;
nD = sqrt(mse(D(W))*k*p);
D = D/nD;

% initialization
[X s R] = initialize(D, W);

% Initial mean shape
M = zeros(k, p);
for i=1:nSample
    M = M + s(i)*R(:, :, i)*D(:, :, i);
end
M = M/nSample;
nM = norm(M(:));
M = M/nM;
s = s/nM;

% Pre-iteration
[X s R M] = preiteration(X, s, R, M, W);

% EM-PND
[X s R M C var_D] = EM_PND(s, R, M, D, W);

% Restore the scale
X = X*nD;
s = s/nD;
var_D = var_D*nD^2;

end

