function [X s R] = initialize(D, W, rrth, K_max)
% function [X s R] = initialize(D, W, rrth, K_max)
%
% Initialize rotations based on orthonormality constraint
%
% Inputs:
%     D: Observations                                 (3 x p x n)
%     W: Weights - observed (true) or missing (false) (3 x p x n)
%     rrth: Threshold for bases
%     K_max: Maximum number of K
%
% Outputs:
%     X: Reconstructed shapes (3 x p x n)
%     s: Scales               (1 x n)
%     R: Rotations            (3 x 3 x n)
%
% This is a custom implementation of the rotation calculation scheme in
% P. F. U. Gotardo and A. M. Martinez,
% "Non-Rigid Structure from Motion with Complementary Rank-3 Spaces,"
% CVPR 2011, Colorado Springs, Colorado, June 20-25, 2011.
%
% Implemented by Minsik Lee (mlee.paper@gmail.com)
% Last update: 2013-08-02


if nargin < 3 || isempty(rrth)
    rrth = 1e-5;
end
if nargin < 4 || isempty(K_max)
    K_max = 10;
end

[k p nSample] = size(D);

s = 1./sqrt(reshape(sum(sum(D.^2, 1), 2), 1, nSample));
Y = reshape(permute(D(1:2, :, :).*repmat(reshape(s, [1 1 nSample]), [2 p]), [1 3 2]), 2*nSample, p);
T = reshape(permute(~W(1:2, :, :), [1 3 2]), 2*nSample, p);

rs = ones(1, nSample);
nmiss = sum(T(:));

d = [ones(2*nSample, 1); zeros(nSample, 1)];

% For-loop : solve rotations for each K
val = inf;
ebit = false;
[U S V] = svd(Y, 'econ');
if nmiss == 0
    ss = cumsum(diag(S).^2);
    ss = ss/ss(end);
end
for i=1:min(floor(p/3), K_max)
    ik = i*k;
    
    % Matrix completion (if necessary)
    if nmiss == 0
        F = U(:, 1:ik);
        ssr = ss(ik);
        if 1-ssr < rrth
            ebit = true;
        end
    else
        F = U(:, 1:ik);
        B = S(1:ik, 1:ik)*V(:, 1:ik)';

        [Y F trs] = mat_comp(Y, T, F, B);
        rs = rs.*trs;
        
        [U S V] = svd(Y, 'econ');
        ss = diag(S).^2;
        ssr = 1/(1+sum(ss(ik+1:end))/sum(ss(1:ik)));
        if 1-ssr < rrth
            ebit = true;
        end
    end

    % Calculate the coefficients of the orthonormality constraint
    temp = reshape(kron(F(1:2:end, :), ones(1, ik)).*kron(ones(1, ik), F(2:2:end, :)), nSample, ik, ik);
    A = [kron(F, ones(1, ik)).*kron(ones(1, ik), F); reshape(temp+permute(temp, [1 3 2]), nSample, ik^2)/sqrt(2)];

    % Initial values for optimization
    if i == 1
        G = eye(k);
    else
        G = [G; zeros(k)];
    end

    % Rectify F
    [G tval] = rect_rot(G, A, d);

    if tval < val
        val = tval;
        mrs = rs;
        mY = Y;
        mF = F;
        mG = G;
    else
        ebit = true;
    end
    
    disp(['Init K = ' num2str(i) ' : rr = ' num2str(1-ssr) ' / cost = ' num2str(tval)]);
    
    if ebit
        break;
    end
end

% Calculate initial parameters
s = s.*mrs;

X = zeros(k, p, nSample);
X(1:2, :, :) = permute(reshape(mY, 2, nSample, p), [1 3 2])./repmat(reshape(s, [1 1 nSample]), [2 p]);

tR = permute(reshape(mF*mG, [], nSample, k), [1 3 2]);
R = zeros(k, k, nSample);
for i=1:nSample
    [U, ~, V] = svd([tR(:, :, i); zeros(1, k)]);
    R(:, :, i) = V*U';
end


end


function [Y F rs] = mat_comp(Y, T, F, B)
% Matrix completion
%
% Y = F*B
% T: a flag matrix for missing entries
%
% Translation components are removed, and scale is normalized for each shape

p = size(Y, 2);

rs = ones(1, size(Y, 1)/2);
pY = ones(sum(T(:)), 1);
while mse(pY-Y(T)) > 1e-10
    pY = Y(T);

    temp = F*B;
    Y(T) = temp(T);
    Y = Y-repmat(mean(Y, 2), 1, p);

    temp = 1./sqrt(sum(sum(reshape(Y.^2, 2, [], p), 1), 3));
    rs = rs.*temp;
    Y = Y.*kron(temp', ones(2, p));

    [B, ~] = qr(B', 0);
    F = Y*B;

    [F, ~] = qr(F, 0);
    B = F'*Y;
end

end


function [G val] = rect_rot(G, A, d)
% Rectify rotations
%
% G: Rectifying transform
% A, d: Coefficients for orthonormality constraint
% val: Value of residual error

ik = size(G, 1);
ikk = numel(G);
k = ikk/ik;
Id = eye(ik);
IdG = eye(ikk);

% Parameters for trust-region optimization
delta = 1;
eta1 = 1/4;
eta2 = 3/4;
kappa = 2;
epsilon = 1e-3;

% Trust-region strategy
f = A*reshape(G*G', [], 1)-d;
c = norm(f)^2;
pc = inf;
while pc-c > 1e-10
    pG = G;
    pf = f;
    pc = c;

    J = A*kron(G, Id)+reshape(permute(reshape(A*kron(Id, G), [], k, ik), [1 3 2]), [], ikk);
    H = J'*J;
    g = J'*f;

    while true
        temp = H+IdG*delta;
        r = (temp+trace(temp)*1e-10*IdG)\g;
        G(:) = G(:)-r;

        f = A*reshape(G*G', [], 1)-d;
        c = norm(f)^2;

        rho = (pc-c)/(pc-norm(pf-J*r)^2);
        rho(isnan(rho)) = 0;
        if rho < eta1 || pc < c
            delta = delta*kappa;
        elseif rho > eta2
            delta = delta/kappa;
        end

        if rho < epsilon || pc < c
            G = pG;
            if delta > 1e10
                c = pc;
                break;
            end
        else
            break;
        end
    end
end

val = mse(f);

end

