function [X s R M] = preiteration(X, s, R, M, W, epsilon, th)
% function [X s R M] = preiteration(X, s, R, M, W, epsilon, th)
%
% Perform pre-iteration to refine initial values
%
% Common:
%     X: Reconstructed shapes (3 x p x n)
%     s: Scales               (1 x n)
%     R: Rotations            (3 x 3 x n)
%     M: Mean shape           (3 x p)
%     
% Inputs:
%     W: Weights - observed (true) or missing (false) (3 x p x n)
%     epsilon: Lower limit for eigenvalue of covariance
%     th: Threshold for iteration
%
% Ref: Minsik Lee, Jungchan Cho, Chong-Ho Choi, and Songhwai Oh,
% "Procrustean Normal Distribution for Non-Rigid Structure from Motion,"
% CVPR 2013, Portland, Oregon, June 23-28, 2013.
%
% Authors: Minsik Lee (mlee.paper@gmail.com), Jungchan Cho (jungchan.cho@gmail.com)
% Last update: 2013-08-08
% License: GPLv3

%
% Copyright (C) 2013 Minsik Lee, Jungchan Cho
% This file is part of NRSfM_PND.
%
% NRSfM_PND is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% NRSfM_PND is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with NRSfM_PND.  If not, see <http://www.gnu.org/licenses/>.


if nargin < 6 || isempty(epsilon)
    epsilon = 1e-7;
end
if nargin < 7 || isempty(th)
    th = 5e-4;
end

[k p nSample] = size(X);
iW = ~W;

G = kron(eye(p)-ones(p)/p, eye(k));
Gv = cell(1, nSample);
for i=1:nSample
    Gv{i} = pinv(G(iW(:, :, i), iW(:, :, i)));
end

sR = zeros(k, k, nSample);
for i=1:nSample
    sR(:, :, i) = s(i)*R(:, :, i);
end
sRX = zeros(k, p, nSample);

fprintf('Pre-iteration...     ');

count = 0;
e_detC = inf;
detC = inf;
while e_detC > th
    PdetC = detC;
    
    % Calculate missing entries
    for i=1:nSample
        d = X(:, :, i);
        RTm = R(:, :, i)'*M;
        h = Gv{i}*(RTm(iW(:, :, i))/s(i) - d(iW(:, :, i)));
                
        tm = X(:, :, i);
        tm(iW(:, :, i)) = tm(iW(:, :, i))+h;
        X(:,:,i) = tm-repmat(mean(tm,2),[1,p]);
    end
    
    % M
    M = zeros(k, p);
    for i=1:nSample   
        M = M+sR(:, :, i)*X(:, :, i);
    end
    M = M/norm(M(:));

    % R, s
    for i=1:nSample
        [U S V] = svd(M*X(:,:,i)','econ');
        R(:,:,i) = U*V';
        s(i) = 1/sum(diag(S));

        sR(:, :, i) = s(i)*R(:, :, i);
        sRX(:, :, i) = sR(:, :, i)*X(:, :, i);
    end
    
    % covergence check
    temp = reshape(sRX, [], nSample)-repmat(M(:), 1, nSample);
    temp = sort(eig(temp*temp'/nSample));
    detC = mean(log(max(temp(2*k+2:end), epsilon))) - log(epsilon);
    e_detC = (PdetC - detC)/detC;    

    count = count + 1;
    fprintf('\b\b\b\b%4d', count);
end

fprintf('\b\b\b\bdone\n');

end

